/****************************************************************************

USB-CAN-Interface Communication Center
Kommunikation mit dem Gert

(c) 2020 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik

RADStudio XE7

Erstellt: 2020-03-11
Letzte nderung: 2020-05-01

*****************************************************************************/

//---------------------------------------------------------------------------

#pragma hdrstop

#include "UnitComm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)

// === TDevComm =============================================================

// Datensatz fr einen ungltigen CAN-Datensatz
const TCANData InvalidCANData = {
	USB_NONE,
	CAN_ID_INVALID,
	0,
	0,
	0,
	0,
	{0,0,0,0,0,0,0,0}
};

//---------------------------------------------------------------------------
__fastcall TDevComm::TDevComm(TUCI *AnUCI, AnsiString ASerialNo):TThread(false)
//---------------------------------------------------------------------------
// Konstruktor
// AnUCI: Gerte-Instanz des ausgewhlten USB-CAN-Interface
// ASerialNo: Seriennummer des zu ffnenden Gertes
{
	Priority = tpHighest;  // sehr hohe Prozessprioritt
	TxQueue = new TList();
	RxQueue = new TList();
	rd = new TCANData;
	TxFlag = false;
	RxFlag = 0;
	CmdFlag = USB_NONE;
	uci = AnUCI;
	SerialNo = ASerialNo;
	FDeviceInfo.cmd = USB_NONE;
	if (uci->Open(SerialNo)) {
		if (!uci->DeviceInfo(&FDeviceInfo)) FDeviceInfo.cmd = USB_NONE;  // Gerteinformationen lesen
	}
}

//---------------------------------------------------------------------------
__fastcall TDevComm::~TDevComm(void)
//---------------------------------------------------------------------------
// Destruktor
{
	if (!Finished) {	// Thread luft noch -> beenden
		Terminate();
		while (!Finished);
	}
	uci->Close();
	// Listen lschen
	TCANData *cd;
	while (TxQueue->Count) {
		cd = (TCANData *)TxQueue->Items[0];
		delete cd;
		TxQueue->Items[0] = NULL;
		TxQueue->Delete(0);
	}
	while (RxQueue->Count) {
		cd = (TCANData *)RxQueue->Items[0];
		delete cd;
		RxQueue->Items[0] = NULL;
		RxQueue->Delete(0);
	}
	delete rd;
}

//---------------------------------------------------------------------------
void __fastcall TDevComm::Execute(void)
//---------------------------------------------------------------------------
// Thread-Routine
{
	TCANData *cd;
	int wait = 0;

	Finished = false;
	try {
		while (!Terminated) {	// "endlos"-Schleife
			if (uci->Opened) {
				// neuer zu sendender Datensatz
				if (TxFlag) {  // neuer Datensatz liegt vor
					TxQueue->Add(td);
					TxFlag = false;  // signalisieren, dass Datensatz bernommen wurde
				}
				// Daten senden
				if (TxQueue->Count) {	// es befinden sich Daten in der Sendewarteschlange
					cd = (TCANData *)TxQueue->Items[0];  // 1. Datensatz senden
					if (uci->CAN_Tx(cd)) {  // senden erfolgreich
						delete cd;	// Datensatz aus dem Heap entfernen
						TxQueue->Items[0] = NULL;
						TxQueue->Delete(0);
					}
					wait = 1;
				}
				// Daten empfangen
				if (uci->CAN_Rx(rd)) {
					RxQueue->Add(rd);
					rd = new TCANData;  // neuen Puffer im Heap fr nchsten Datenempfang anlegen
					wait = 1;
				}
				// bereit gestellte und ausgelesene Daten behandeln
				if (RxFlag==1) {	// bereitgestellte Daten wurden ausgelesen und knnen gelscht werden
					delete FReceived;
					FReceived = NULL;
					RxQueue->Delete(0);
					RxFlag = 0;  // keine Daten bereit gestellt
				}
				// Bereitstellung empfangener Daten
				if ((RxFlag==0) && (RxQueue->Count)) {  // keine Daten bereit gestellt und neue verfgbar
					FReceived = (TCANData *)RxQueue->Items[0];
					RxFlag = 2;  // signalisieren, dass Daten zum Abruf bereit stehen
				}
				// andere Kommandos ausfhren
				switch (CmdFlag) {
				case USB_CONFIG_W:	// neue Konfiguration setzen
					uci->SetDeviceConfig(&FDeviceConfig);
					CmdFlag = USB_NONE;
					wait = 5;
					break;
				case USB_CAN_FILTER:  // Filter setzen
					uci->CAN_SetFilter(&FCANFilter);
					CmdFlag = USB_NONE;
					wait = 5;
					break;
				case USB_CLR_STATUS:  // Status zurck setzen und FIFO-Puffer lschen
					uci->ClearStatus(true,true,true);
					CmdFlag = USB_NONE;
					wait = 1;
					break;
				default:  // ungltiges Kommando
					CmdFlag = USB_NONE;
				}
			}
			Sleep(wait);
		}
	} catch (Exception &exception) {
	}
	Finished = true;
}

//---------------------------------------------------------------------------
void __fastcall TDevComm::Transmit(TCANData *ACanData)
//---------------------------------------------------------------------------
// Zu sendende Daten in die Sendewarteschlange ablegen
// ACanData muss auf dem Heap angelegt worden sein!
{
	td = ACanData;  // Zwischenpuffer
	TxFlag = true;	// Thread signalisieren, dass neuer Datensatz vorliegt
	while (TxFlag);  // warten, bis der Thread den Datensatz bernommen hat
}

//---------------------------------------------------------------------------
bool __fastcall TDevComm::Receive(TCANData *ACanData)
//---------------------------------------------------------------------------
// Empfangene Daten bertragen
// Ergebnis true: ACanData wurde mit gltigen Daten befllt
//         false: keine Daten in ACanData berragen
{
	bool r = false;
	if (RxFlag==2) {	// es stehen neue Daten zum Abruf bereit
		memcpy_s(ACanData,sizeof(TCANData),FReceived,sizeof(TCANData));
		RxFlag = 1;  // Thread signalisieren, dass er die Daten lschen kann
		r = true;
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TDevComm::SetDeviceConfig(TDeviceConfig *ADeviceConfig)
//---------------------------------------------------------------------------
// Neue Gertekonfiguration setzen
{
	while (CmdFlag!=USB_NONE);  // warten, bis ein anderes Kommando ausgefhrt ist
	memcpy_s(&FDeviceConfig,sizeof(TDeviceConfig),ADeviceConfig,sizeof(TDeviceConfig));
	CmdFlag = USB_CONFIG_W;
}

//---------------------------------------------------------------------------
void __fastcall TDevComm::SetFilter(TCANFilter *ACANFilter)
//---------------------------------------------------------------------------
// CAN-Filter setzen
{
	while (CmdFlag!=USB_NONE);  // warten, bis ein anderes Kommando ausgefhrt ist
	memcpy_s(&FCANFilter,sizeof(TCANFilter),ACANFilter,sizeof(TCANFilter));
	CmdFlag = USB_CAN_FILTER;
}

//---------------------------------------------------------------------------
void __fastcall TDevComm::ClearStatus(void)
//---------------------------------------------------------------------------
// Status zurck setzen
{
	while (CmdFlag!=USB_NONE);  // warten, bis ein anderes Kommando ausgefhrt ist
	CmdFlag = USB_CLR_STATUS;
}

