/****************************************************************************

Demo for the USB/CAN Interface device

RADStudio XE7

(c) 2020 Reusch Elektronik

Created: 2020-03-27
Changed: 2020-05-01

****************************************************************************/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;

//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
	: TForm(Owner)
//---------------------------------------------------------------------------
{
	//
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
//---------------------------------------------------------------------------
// Initialization
{
	if (!USB_Initialize()) {	// driver not installed
		MessageBeep(16);
		MessageDlg("LibUSB-Win32 driver not installed!",mtError,TMsgDlgButtons() << mbCancel,0);
		Close();
	} else {
		uci = new TUCI();	// generate an instance of the interface class
		uci->ScanForDevices(ComboBoxDevices->Items); // fill combobox with available devices
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormClose(TObject *Sender, TCloseAction &Action)
//---------------------------------------------------------------------------
{
	//
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::FormDestroy(TObject *Sender)
//---------------------------------------------------------------------------
// final actions
{
	delete uci;
}

//---------------------------------------------------------------------------
MESSAGE void __fastcall TFormMain::WMDeviceChange(TMessage &Message)
//---------------------------------------------------------------------------
// A device was connected or removed (handle of the according Windows message)
// Check, if the used device is still present
{
	if (uci) {
		if ((uci->Opened) && (uci->DeviceRemoved())) {  // the used device was opened and is removed!
			Memo1->Lines->Add("device removed!");
			// Hint: the device is closed automatically by calling "DeviceRemoved()", if it is no longer connected
		}
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::MsgToMemo(TCANData *CANData)
//---------------------------------------------------------------------------
// View a CAN message in the memo
{
	UnicodeString s;
	if (CANData->cmd==USB_CAN_TX) s = "tx: ";
	else s = "rx: ";
	if (CANData->IdType==CAN_ID_STD) s = s+"std ";
	else  s = s+"ext ";
	if (CANData->RemoteRequest) s = s + "(remote request DLC="+IntToStr(CANData->DataLength)+")";
	else
		for (int i=0; i<CANData->DataLength; i++) s = s+" 0x"+IntToHex((int)CANData->Data[i],2);
	Memo1->Lines->Add(s);

}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonOpenClick(TObject *Sender)
//---------------------------------------------------------------------------
// open the USB communication
// ComboBoxDevices contains the serial numbers of all found devices.
// If no entry is chosen, the first will be used.
{
	Memo1->Clear();

	// open
	if (uci->Open(ComboBoxDevices->Text)) {  // opening device succeeded
		Memo1->Lines->Add("device opened");
	} else {
		Memo1->Lines->Add("opening failed");
	}

	// retrieve device information
	const AnsiString RevisionString[4] = {"1.0","?","?","?"};
	TDeviceInfo di;
	if (uci->DeviceInfo(&di)) {	// retrieving device info succeeded
		AnsiString sn = di.SerialNo;
		Memo1->Lines->Add(" serial number: "+sn);
		Memo1->Lines->Add(" revision: "+RevisionString[di.Revision]);
		Memo1->Lines->Add(" version: V"+IntToHex(di.Version>>8,2)+"."+IntToHex(di.Version & 0xFF,2));
	}

	// retrieve device configuration
	TDeviceConfig dc;
	if (uci->GetDeviceConfig(&dc)) {	// retrieving device configuration succeeded
		ComboBoxSpeed->ItemIndex = dc.Speed;
		CheckBoxTermination->Checked = dc.Termination;
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonCloseClick(TObject *Sender)
//---------------------------------------------------------------------------
// close the USB communication
{
	uci->Close();
	Memo1->Lines->Add("device closed");
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonConfigClick(TObject *Sender)
//---------------------------------------------------------------------------
// set a new device configuration
{
	TDeviceConfig dc;
	dc.Speed = ComboBoxSpeed->ItemIndex;
	dc.Termination = CheckBoxTermination->Checked;
	dc.Retransmit = 1;
	dc.LoopBackMode = 0;
	if (uci->SetDeviceConfig(&dc)) {	// setting new device configuration succeeded
		Memo1->Lines->Add("configuration set");
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonTxClick(TObject *Sender)
//---------------------------------------------------------------------------
// transmit a few messages
{
	TCANData cd;
	// a standard message
	cd.IdType = CAN_ID_STD;
	cd.RemoteRequest = CAN_RTR_DATA;
	cd.Identifier = 0x123;
	cd.DataLength = 2;
	cd.Data[0] = 0x0F;
	cd.Data[1] = 0x1E;
	if (uci->CAN_Tx(&cd)) MsgToMemo(&cd);
	else Memo1->Lines->Add("tx failed");
	// an extended message
	cd.IdType = CAN_ID_EXT;
	cd.RemoteRequest = CAN_RTR_DATA;
	cd.Identifier = 0x1234;
	cd.DataLength = 3;
	cd.Data[0] = 0x0F;
	cd.Data[1] = 0x1E;
	cd.Data[2] = 0x2D;
	if (uci->CAN_Tx(&cd)) MsgToMemo(&cd);
	else Memo1->Lines->Add("tx failed");
	// a standard remote request
	cd.IdType = CAN_ID_STD;
	cd.RemoteRequest = CAN_RTR_REMOTE;
	cd.Identifier = 0x123;
	cd.DataLength = 2;
	if (uci->CAN_Tx(&cd)) MsgToMemo(&cd);
	else Memo1->Lines->Add("request failed");
	// hint: a transmission fail is usually caused by a full transmission FIFO buffer (or device is closed)
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonRxClick(TObject *Sender)
//---------------------------------------------------------------------------
// retrieve received messages
{
	TCANData cd;
	while (uci->CAN_Rx(&cd)) {	// read all received messages from the device FIFO buffer
		MsgToMemo(&cd);
	}
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonFilterMaskClick(TObject *Sender)
//---------------------------------------------------------------------------
// set a masked filter
// in this example filter bank 0 is overwritten
// (by power on default this bank is activated and will pass all messages)
// only standard messages in the range from 0x120 to 0x123 will pass
{
	TCANFilter cf;
	cf.Bank = 0;
	cf.Active = 1;
	cf.Mode = CAN_FILTERMODE_IDMASK;
	cf.IdType = CAN_ID_STD;
	cf.ID = 0x120;  // the message ID base
	cf.Mask = 0x12C; // the relevant bits of the message ID (determines the range)
	if (uci->CAN_SetFilter(&cf)) Memo1->Lines->Add("1st filter example");
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonFilterListClick(TObject *Sender)
//---------------------------------------------------------------------------
// set a list filter
// in this example filter bank 1 is used
// only the two extended messages with the ID 0x1234 and 0x234A will pass
{
	TCANFilter cf;
	cf.Bank = 1;
	cf.Active = 1;
	cf.Mode = CAN_FILTERMODE_IDLIST;
	cf.IdType = CAN_ID_EXT;
	cf.ID1 = 0x1234;  // 1st message ID
	cf.ID2 = 0x234A;  // 2nd message ID
	if (uci->CAN_SetFilter(&cf)) Memo1->Lines->Add("2nd filter example");
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonFilterAllClick(TObject *Sender)
//---------------------------------------------------------------------------
// set filter bank 0 to power on default -> all messages will pass
{
	if (uci->CAN_SetFilterAll()) Memo1->Lines->Add("all messages will pass");
}

//---------------------------------------------------------------------------
