/******************************************************************************
 * uci_common.h
 *
 * Common Agreements Firmware / Windows Application
 *
 * Created: 2020-02-02
 * Changed: 2020-05-01
 * Author:  Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik, www.reusch-elektronik.de
 ******************************************************************************/

#ifndef COMMON_H_
#define COMMON_H_

#include <stdint.h>

//=============================================================================
// Platform specific
//=============================================================================
#ifdef WIN32
#define WINDOWS
#endif
#ifdef WIN64
#define WINDOWS
#endif
#ifdef _BCB
#define WINDOWS
#endif

#ifdef WINDOWS
#define xchar AnsiChar
#define USB_FS_MAX_PACKET_SIZE 64
#else
#include <stdbool.h>
#define xchar char
#endif

//=============================================================================
// USB Communication
//=============================================================================

// Vendor and Product ID
#define USBD_VID                    0x20A0	// obdev.at
#define USBD_PID                    0x41E8	// PID for USB-CAN-Interface (LibUSB)

// endpoints (see also usbd_usr.h)
#define IN_EP						0x82  	// EP2 for data IN
#define OUT_EP  					0x01  	// EP1 for data OUT

// USB Commands
#define USB_NONE					0x0000	// no comand
#define USB_CLOSE					0x0100	// close USB communication (hence no commands except USB_OPEN will be accepted)
#define USB_OPEN					0x0101	// open USB Communication (hence other commands will be accepted)
#define USB_INFO                    0x0200  // device information
#define USB_STATUS                  0x0300  // status request
#define USB_CLR_STATUS              0x0301  // clear status and FIFO buffers
#define USB_CONFIG_R                0x0400  // request device configuration
#define USB_CONFIG_W                0x0401  // set a new device configuration
#define USB_CAN_RX                  0x0500  // transfer received CAN messages to host (see TCANData)
#define USB_CAN_TX                  0x0501  // output messages on the CAN interface (see TCANData)
#define USB_CAN_FILTER              0x0511  // set a CAN filter
#define USB_CAN_FILTER_ALL          0x0521  // set CAN filter bank 0 to receive all messages (power on default)
#define USB_ACK                     0x0600  // acknowledge
#define USB_TEST                    0xFF00  // for test purposes, do not use!

typedef uint16_t TCmd;

// command with parameter
typedef struct {
	TCmd cmd;					    // USB command
	uint16_t param;					// parameter (depending on the command)
} TCommand;

#define VERSIONSTRING "VersionString"	// used in the firmware to mark the position of the version information

// device information
typedef struct {
	TCmd cmd;					    // USB command (USB_INFO)
	uint8_t Reserved1;
    uint8_t  Revision;              // Board-Revision (0..3, 0 = Rev. 1.0)
    uint16_t Version;               // firmware version (BCD coded)
    uint16_t Reserved2;
    xchar SerialNo[16];             // serial number (= C serial no.)
    uint32_t Reserved3[10];
} TDeviceInfo;

// device configuration
#define SPEED_10K   0               // 10kBit/s
#define SPEED_20K   1               // 20kBit/s
#define SPEED_50K   2               // 50kBit/s
#define SPEED_100K  3               // 100kBit/s
#define SPEED_125K  4               // 125kBit/s
#define SPEED_250K  5               // 250kBit/s
#define SPEED_500K  6               // 500kBit/s
#define SPEED_1M    7               // 1MBit/s
#define NUM_SPEEDS  8               // number of supported speeds
#define SPEEDVAL uint16_t SpeedVal[NUM_SPEEDS] = {10,20,50,100,125,250,500,1000};
#define SPEED_DEFAULT SPEED_10K     // speed after reset (power on default)
#define TERMINATION_OFF 0           // no bus termination
#define TERMINATION_ON  1           // bus termination active
typedef struct {
	TCmd cmd;					    // USB command (USB_CONFIG_R or USB_CONFIG_W)
	uint8_t Speed;                  // index of the CAN bus transfer rate (SPEED_ constant)
    uint8_t Termination;            // terminiation off/on (TERMINATION_ constant)
    uint8_t Retransmit;             // 1 = automatic retransmission at failure (default)
    uint8_t LoopBackMode;           // 1 = loop back modus (for debug purposes)
    uint8_t Reserved1;
    uint8_t Reserved2;
} TDeviceConfig;

// CAN data exchange between device and host
#ifdef WINDOWS  // redeclarations for Windows
#define CAN_ID_STD      0       // standard CAN message
#define CAN_ID_EXT      4       // extended CAN message
#define CAN_RTR_DATA    0       // data frame
#define CAN_RTR_REMOTE  2       // remote request frame
#endif
#define CAN_ID_INVALID  8       // data set doesn't contain valid data (relevant for receive only)
typedef struct {
    TCmd cmd;                    // USB command (USB_CAN_TX or USB_CAN_RX)
    uint8_t  IdType;             // std. or ext. message (CAN_ID_ constant)
    uint8_t  RemoteRequest;      // data or remote request frame (CAN_RTR_ constant)
    uint8_t  Reserved;
    uint8_t  DataLength;         // number of valid bytes in Data (0..8)
    uint32_t Identifier;         // CAN identifier for the CAN message
    uint8_t  Data[8];            // data (payload)
} TCANData;

// set a CAN filter
#ifdef WINDOWS  // redeclarations for Windows
#define CAN_FILTERMODE_IDMASK   0       // filter definition by bit mask
#define CAN_FILTERMODE_IDLIST   1       // defintion of two IDs (which will pass the filter)
#endif
typedef struct {
    TCmd cmd;                    // USB command (USB_CAN_FILTER)
    uint16_t Reserved1;
    uint8_t IdType;              // std. or ext. message ID (CAN_ID_ constant)
    uint8_t Bank;                // filter bank number (0..13)
    uint8_t Active;              // filter bank inactive (0) or active (1)
    uint8_t Mode;                // filter mode (CAN_FILTERMODE_ constant)
    union {
        struct {    // parameter for mode "mask"
            uint32_t ID;         // ID template
            uint32_t Mask;       // mask (relevant bits of ID)
        };
        struct {    // parameter for mode "list"
            uint32_t ID1;        // 1st ID
            uint32_t ID2;        // 2nd ID
        };
    };
} TCANFilter;

// acknowledge (device to host)
#define ACK_FAILED  0            // failed
#define ACK_OK      1            // successful
typedef struct {
    TCmd cmd;                    // USB command (USB_ACK)
    uint16_t ack;                // contains an ACK_ constant
} TAcknowledge;

//-----------------------------------------------------------------------------
// status and error information
//-----------------------------------------------------------------------------

// possible combinations of "Status"
#define STATUS_NONE			    0x00000000	// initial value
#define STATUS_INITIALIZED      0x00000001  // Initialization completed
#define STATUS_USB_INITIALIZED  0x00000002  // USB initialized
#define STATUS_USB_ADDRESS      0x00000004  // the device got an USB address
#define STATUS_USB_MSG          0x00000008  // USB address submitted
#define STATUS_USB_OPENED       0x00000010  // USB communication opened
#define STATUS_LED              0x00000020  // USB status has to be checked
#define STATUS_CAN_TX           0x00000400  // data transmission on the CAN interface
#define STATUS_WATCHDOG         0x00000800  // reset caused by watchdog timeout
#define STATUS_READERROR        0x80000000  // used in the Windows application

typedef struct {
    TCmd cmd;                   // USB command (USB_STATUS)
	uint16_t Reserved;
	uint32_t Status;			// status with status bits (see STATUS_ constants)
	uint32_t RxLost;            // number of lost CAN messages at receiving (receiving buffer full)
	uint32_t TxLost;            // number of lost CAN messages at transmission (transfer buffer full)
	uint32_t TxError;           // number of CAN messages, who couldn't be transmitted, caused by a transmission error
} TStatus;

#endif /* COMMON_H_ */
