/****************************************************************************

Basic class for user defined USB devices
based on LibUSB-Win32 driver

The interface to the USB driver wil be linked dynamically

(c) 2019-2020 Reusch Elektronik

Created: 2019-01-17
Changed: 2020-03-27

****************************************************************************/

#pragma hdrstop

#include <Math.hpp>
#include "usbdevice_libdyn.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

// === General USB Routines =================================================

bool Initialized = false;

//---------------------------------------------------------------------------
bool USB_Initialize(void)
//---------------------------------------------------------------------------
// Initialize USB communication
// Returns fals, if LibUSB driver not installed or DLL not found.
{
	bool r = false;
	TFileName fn;
	wchar_t Path[1024];

	// check, if LibUSB driver is installed
	GetSystemDirectory((wchar_t *)&Path,sizeof(Path));
	AnsiString s = Path;
	s = s+"\\libusb0.dll";
	r = (FileExists(s));

	if (r) r = usb_init();
	Initialized = r;
	return r;
}

//------------------------------------------------------------------------------
int Scan(uint16_t VendorID, uint16_t ProductID, TStrings *DeviceList)
//------------------------------------------------------------------------------
// Search for connected USB devices with the specified vendor and product ID.
// Returns number of found devices.
// The serial numbers of the found devices will signed into DeviceList.
// Hint: DeviceList has to be instanced before calling!
// The list will be cleared before scanning.
{
	struct usb_bus *bus;
	struct usb_device *dev;
	struct usb_dev_handle *hdev; // USB device handle
	char buf[256];
	int n = 0;

	if (DeviceList==NULL) return 0;  // list is not instanced
	DeviceList->Clear();

	usb_init();         // initialize the library
	usb_find_busses();  // find all busses
	usb_find_devices(); // find all connected devices
	// open
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if ((dev->descriptor.idVendor==VendorID)&&(dev->descriptor.idProduct==ProductID)) {
				hdev = usb_open(dev);
				if (hdev) {
					usb_get_string_simple(hdev,dev->descriptor.iSerialNumber,buf,sizeof(buf)); // get serial number
					if (buf[0]) {
						n++;
						DeviceList->Add(buf);	// device has a serial number
					}
					usb_close(hdev);
					hdev = NULL;
				}
			}
		}
	}
	return n;
}

//---------------------------------------------------------------------------
struct usb_dev_handle* OpenDevice(uint16_t VendorID, uint16_t ProductID, AnsiString *SerialNo)
//---------------------------------------------------------------------------
// Open the device communication
// VendorID: The vendor ID of the device to open
// ProductID: The product ID of the device to open
// SerialNo: Serial number of the device to open
//           This string can be leaved empty. In this case, the first found
//           device will be opened and the serial number of this device will
//           be returned in this variable.
// Returns a pointer to USB information or 0, if failed.
{
	struct usb_bus *bus;
	struct usb_device *dev;
	struct usb_dev_handle *hdev = NULL;
	int j;
	char buf[256];
	bool done = false;

	// search devices
	usb_find_busses();  // find all busses
	usb_find_devices(); // find all connected devices
	// open
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if ((!done)&&(dev->descriptor.idVendor==VendorID)&&(dev->descriptor.idProduct==ProductID)) {
				hdev = usb_open(dev);
				if (hdev) {
					if (SerialNo->Length()) {  // open device with specific S/N
						j = dev->descriptor.iSerialNumber;  // string index for serial number
						usb_get_string_simple(hdev,j,buf,sizeof(buf)); // get serial number
						done = (*SerialNo==buf);
					} else {
						done = true; // no S/N: accept first found device
						j = dev->descriptor.iSerialNumber;  // string index for serial number
						usb_get_string_simple(hdev,j,buf,sizeof(buf)); // get serial number
						*SerialNo = buf;  // return the serial number of the used device
					}
					if (!done) {
						usb_close(hdev);
						hdev = 0;
					}
				}
			}
		}
	}
	if (hdev) {
		usb_set_configuration(hdev, 1);  // select 1st USB configuration
		usb_claim_interface(hdev, 0);  // select interface 0
	}
	return hdev;
}

//---------------------------------------------------------------------------
void CloseDevice(struct usb_dev_handle *hdev)
//---------------------------------------------------------------------------
// Close the USB communication to the device
{
	if (hdev) {
		usb_release_interface(hdev, 0);  // close pipe
		usb_close(hdev);
		hdev = NULL;
	}
}

// === USB-Gerte-Basisklasse ===============================================

//---------------------------------------------------------------------------
__fastcall TUSBDevice::TUSBDevice(void)
//---------------------------------------------------------------------------
// Constructor
{
	// initialization
	hdev = NULL;
	IsReading = false;
	IsWriting = false;
	FOpened = false;
	FVendorID = 0;
	FProductID = 0;
	FSerialNumber = "";
	FLastError = 0;
	FErrorCountWrite = 0;
	FErrorCountRead = 0;
}

//---------------------------------------------------------------------------
__fastcall TUSBDevice::~TUSBDevice(void)
//---------------------------------------------------------------------------
// Destructor
{
	if (FOpened) Close();
}

//---------------------------------------------------------------------------
int __fastcall TUSBDevice::ScanForDevices(TStrings *DeviceList)
//---------------------------------------------------------------------------
// Scan USB bus for devices (with S/N) with the specified product and vendor ID
// The serial number of the found devices will be filed into the list.
// The properties VID and PID has to be specified accordingly.
// DeviceList has to be instanced before calling!
// Returns number of found devices (= number of list entries)
{
	if (!Initialized) USB_Initialize();
	if ((Initialized) && (FVendorID) && (FProductID)) return Scan(FVendorID,FProductID,DeviceList);
	else return 0;
}

//---------------------------------------------------------------------------
bool __fastcall TUSBDevice::DeviceRemoved(void)
//---------------------------------------------------------------------------
// Check, if the opened device has to be removed (unplugged)
// Device connection has to be opened (otherwise the result is always false).
// The device connection wil be closed automatically, if the device is no
// longer present.
// Returns true, if device no longer present.
{
	if (FOpened) {
		bool found = false;
		TStringList *dl = new TStringList();
		int n = ScanForDevices(dl);
		if (n) {
			for (int i=0; i<n; i++) {
				found = found || (dl->Strings[i]==FSerialNumber);
			}
		}
		delete dl;
		if (!found) Close();  // close device
		return !found;
	} else return false;
}

//---------------------------------------------------------------------------
uint32_t __fastcall TUSBDevice::CRC32WideFast(uint8_t *Buffer, unsigned int Size)
//---------------------------------------------------------------------------
// Calculation of the CRC32 checksum of a data block
// (compatible with the CRC32 checksum calculation in a STM32)
// Buffer: pointer to the buffer with data for CRC32 calculation
// Size: size of the buffer
// Important: Size has to be a multiple of 4!
{
	static const unsigned int CrcTable[16] = { // Nibble lookup table for 0x04C11DB7 polynomial
	0x00000000,0x04C11DB7,0x09823B6E,0x0D4326D9,0x130476DC,0x17C56B6B,0x1A864DB2,0x1E475005,
	0x2608EDB8,0x22C9F00F,0x2F8AD6D6,0x2B4BCB61,0x350C9B64,0x31CD86D3,0x3C8EA00A,0x384FBDBD };

	uint32_t Crc = 0xFFFFFFFF;
	Size = Size >> 2;
	while(Size--)
	{
		Crc = Crc ^ *((uint32_t *)Buffer); // Apply all 32-bits
		Buffer += 4;
		// Process 32-bits, 4 at a time, or 8 rounds
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28]; // Assumes 32-bit reg, masking index to 4-bits
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28]; // 0x04C11DB7 Polynomial used in STM32
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
		Crc = (Crc << 4) ^ CrcTable[Crc >> 28];
	}
	return(Crc);
}

//---------------------------------------------------------------------------
bool __fastcall TUSBDevice::Open(AnsiString SerialNo)
//---------------------------------------------------------------------------
// Open the device communication
// SerialNo: Serial number of the device to open
//           can be left empty to open the first found device
// Returns true if succeeded.
{
	AnsiString sn = SerialNo;
	if (!Initialized) USB_Initialize();
	if ((!FOpened) && (Initialized) && (FVendorID) && (FProductID)) {
		hdev = OpenDevice(FVendorID,FProductID,&sn);
		if (hdev) {
			if (SerialNo.Length()) FSerialNumber = SerialNo;  // remind specified S/N
			else FSerialNumber = sn;  // no S/N specified -> OpenDevice() returns it
			FOpened = true;
		}
		return FOpened;
	}	return false;
}

//---------------------------------------------------------------------------
void __fastcall TUSBDevice::Close(void)
//---------------------------------------------------------------------------
// Close device communication
{
	if (FOpened) {
		CloseDevice(hdev);
		hdev = NULL;
		FOpened = false;
	}
}

//---------------------------------------------------------------------------
int __fastcall TUSBDevice::Write(byte *Buffer, int Size, int Timeout)
//---------------------------------------------------------------------------
// Transmit data to the device
// Buffer: pointer to data to transmit
// Size: data size
// Timeout: timeout value in milliseconds
// Result: number of transmitted bytes (usually size) or a negative error code
{
#define WRITE_RETRIES 3  // number of retries until abort
	int r = 0, n = WRITE_RETRIES;
	if ((FOpened) && (!IsWriting)) {
		IsWriting = true;
		do {
			r = usb_bulk_write(hdev,0x01,Buffer,Size,Timeout);
			if (r<0) {
				FLastError = r;
				FErrorCountWrite++;
				n--;
			}
		} while ((r<0) && (n>0));
		IsWriting = false;
	}
	return r;
}

//---------------------------------------------------------------------------
int __fastcall TUSBDevice::Read(byte *Buffer, int Size, int Timeout)
//---------------------------------------------------------------------------
// Read data from device
// Buffer: pointer to a buffer, which will take the read data
// Size: data size
// Timeout: timeout value in milliseconds
// Result: number of read bytes or a negative error code
{
	int r = 0;
	if ((FOpened) && (!IsReading)) {
		IsReading = true;
		r = usb_bulk_read(hdev,0x82,Buffer,Size,Timeout);
		if (r<0) {
			FLastError = r;
			FErrorCountRead++;
			if (r!=-116) { // error caused by timeout
				usb_resetep(hdev,0x82);  // reset endpoint
				usb_bulk_read(hdev,0x82,Buffer,Size,Timeout);  // read remaining data (may be wrong)
			}
		}
		IsReading = false;
	}
	return r;
}

//---------------------------------------------------------------------------
void __fastcall TUSBDevice::ResetIn(void)
//---------------------------------------------------------------------------
// Reset IN-Endpoint
{
	if (FOpened) {
		usb_resetep(hdev,0x82);
	}
}


